﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Battleship.Entities;
using Battleship.MovementStrategy;

namespace Battleship.DrawningObjects
{


    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawningShip
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityShip? EntityShip { get; protected set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// Левая координата прорисовки 
        /// </summary>
        protected int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки 
        /// </summary>
        protected int _startPosY;
        /// <summary>
        /// Ширина прорисовки 
        /// </summary>
        protected readonly int _buttleshipWidth = 175;
        /// <summary>
        /// Высота прорисовки 
        /// </summary>
        protected readonly int _buttleshipHeight = 80;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет основы</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <returns>true - объект создан, false - проверка не пройдена, 
        ///нельзя создать объект в этих размерах</retu rns>

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawningCar
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawningObjectShip(this);

        public DrawningShip(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _buttleshipWidth || height < _buttleshipHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityShip = new EntityShip(speed, weight, bodyColor);

        }
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param> 
        /// <param name="buttleshipWidth">Ширина прорисовки автомобиля</param>
        /// <param name="buttleshipHeight">Высота прорисовки автомобиля</param>
        protected DrawningShip(int speed, double weight, Color bodyColor, int
        width, int height, int buttleshipWidth, int buttleshipHeight)
        {
            if (width <= _buttleshipWidth || height <= _buttleshipHeight)
                return;
            _pictureWidth = width;
            _pictureHeight = height;
            _buttleshipWidth = buttleshipWidth;
            _buttleshipHeight = buttleshipHeight;
            EntityShip = new EntityShip(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x >= 0 && x + _buttleshipWidth <= _pictureWidth && y >= 0 && y + _buttleshipHeight <= _pictureHeight)
            {
                _startPosX = x;
                _startPosY = y;
            }
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
      
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityShip == null)
            {
                return;
            }
            Pen pen = new(Color.Black, 2);

            //основа
            Brush mainBrush = new SolidBrush(EntityShip.BodyColor);
            Point[] hull = new Point[]
            {
                new Point(_startPosX + 5, _startPosY + 0),
                new Point(_startPosX + 120, _startPosY + 0),
                new Point(_startPosX + 160, _startPosY + 35),
                new Point(_startPosX + 120, _startPosY + 70),
                new Point(_startPosX + 5, _startPosY + 70),
            };
            g.FillPolygon(mainBrush, hull);
            g.DrawPolygon(pen, hull);

            //блоки
            Brush blockBrush = new
            SolidBrush(Color.DimGray);
            g.FillRectangle(blockBrush, _startPosX + 70, _startPosY + 15, 20, 40);
            g.DrawRectangle(pen, _startPosX + 70, _startPosY + 15, 20, 40);

            g.FillRectangle(blockBrush, _startPosX + 40, _startPosY + 25, 30, 20);
            g.DrawRectangle(pen, _startPosX + 40, _startPosY + 25, 30, 20);

            g.FillEllipse(blockBrush, _startPosX + 100, _startPosY + 20, 30, 30);
            g.DrawEllipse(pen, _startPosX + 100, _startPosY + 20, 30, 30);

            //для ускорения 
            Brush speedBrush = new
            SolidBrush(Color.Gold);
            g.FillRectangle(speedBrush, _startPosX + 0, _startPosY + 10, 5, 20);
            g.DrawRectangle(pen, _startPosX + 0, _startPosY + 10, 5, 20);
            g.FillRectangle(speedBrush, _startPosX + 0, _startPosY + 40, 5, 20);
            g.DrawRectangle(pen, _startPosX + 0, _startPosY + 40, 5, 20);
        }
        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;
        /// <summary>
        /// Координата Y объекта
        /// /// </summary>
        public int GetPosY => _startPosY;
        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _buttleshipWidth;
        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _buttleshipHeight;
        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityShip == null)
            {
                return false;
            }
            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityShip.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityShip.Step > 0,
                //вправо
                DirectionType.Right => _startPosX + EntityShip.Step + _buttleshipWidth < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + EntityShip.Step + _buttleshipHeight < _pictureHeight,
                _ => false,
            };
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityShip == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityShip.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityShip.Step;
                    break;
                // вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityShip.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityShip.Step;
                    break;
            }
        }
    }
}



